package gs.hex.v0

/** Type Class for encoding types as hexadecimal strings.
  */
trait HexEncode[A]:
  /** Encode the given data as a hexadecimal string.
    *
    * @param data
    *   The data to encode.
    * @return
    *   The encoded data.
    */
  def toHexString(data: A): String

  extension (data: A) def toHex(): String = toHexString(data)

object HexEncode:

  /** Retrieve the [[HexEncode]] instance for the given type.
    *
    * @param H
    *   The type.
    * @return
    *   The [[HexEncode]] instance for the given type.
    */
  def apply[A](
    using
    H: HexEncode[A]
  ): HexEncode[A] = H

  given HexEncode[Array[Byte]] = new HexEncode[Array[Byte]] {

    /** @inheritDoc
      */
    def toHexString(data: Array[Byte]): String = Hex.toHexString(data)
  }

  given HexEncode[String] = new HexEncode[String] {

    /** @inheritDoc
      */
    def toHexString(data: String): String = Hex.toHexString(data.getBytes())
  }

  given HexEncode[Boolean] = new HexEncode[Boolean] {

    /** @inheritDoc
      */
    def toHexString(data: Boolean): String = if data then "1" else "0"
  }

  given HexEncode[Int] = new HexEncode[Int] {

    /** @inheritDoc
      */
    def toHexString(data: Int): String =
      val bb = java.nio.ByteBuffer.allocate(4)
      bb.putInt(data)
      Hex.toHexString(bb.array())

  }

  given HexEncode[Long] = new HexEncode[Long] {

    /** @inheritDoc
      */
    def toHexString(data: Long): String =
      val bb = java.nio.ByteBuffer.allocate(8)
      bb.putLong(data)
      Hex.toHexString(bb.array())

  }

end HexEncode
